@@ -470,6 +470,15 @@ void test1(void)
         "    <page>bothpage.com</page>\n"
         "    <link>bothlink.com</link>\n"
         "  </skiplink>\n"
+        "  <skiprequestheader>\n"
+        "    <page>omit4.com/s.*</page>\n"
+        "    <header>X-Forwarded-Host</header>\n"
+        "  </skiprequestheader>\n"
+        "  <addrequestheader>\n"
+        "    <page>omit4.com/s.*</page>\n"
+        "    <header>X-Request-Header</header>\n"
+        "    <value>Added by cproxy</value>\n"
+        "  </addrequestheader>\n"
         "</filter>\n";
     make_cf_config("./cf-config.unittest.cfg", // config file to write
                    "localhost:9036/XXX/node102",  // url prefix
@@ -483,6 +492,10 @@ void test1(void)
     YAZ_CHECK( check_response_code(pack, 400) ); 
     YAZ_CHECK( check_response_content(pack,
                                 "No cproxy session found in the URL" ) );
+    YAZ_CHECK( check_response_header(pack,"X-MK-Component", " ") );
+      // The header should contain hostname and version, but those can vary. So
+      // we only check the space between them. That is enough to see that the 
+      // header line is there.
     delete(pack);
 
     // No proxy host or prefix in the URL. Enabling debug. 
@@ -550,6 +563,7 @@ void test1(void)
                                         ".* skiplink.com" ));
     YAZ_CHECK( check_any_request_header(pack,"X-Metaproxy-SkipLink",
                                         "bothpage.com bothlink.com" ));
+    YAZ_CHECK( check_no_request_header(pack,"X-MK-Component") );
     delete(pack);
 
     // Existing session file, but pretty much emtpy.
@@ -639,6 +653,7 @@ void test1(void)
     YAZ_CHECK( check_response_code(pack, 302) );
     YAZ_CHECK( check_response_header(pack, "Location",
                    "http://localhost:9036/XXX/node102/100/ref.host/page.txt") );
+    YAZ_CHECK( check_response_header(pack,"X-MK-Component", " ") );
     delete(pack);
     
     // simple referer header with just a host name, no trailing slash
@@ -708,7 +723,7 @@ void test1(void)
           "http://localhost:9036/XXX/node102/100/"
           "ref.host/path/to/page.txt?param=value") );
     delete(pack);
-    
+
     // No referer header, fall back to a cproxy session cookie
     pack = make_package("http://localhost:9036/cproxydebug-15/"
                "path/to/page.txt",1);
@@ -735,6 +750,23 @@ void test1(void)
     YAZ_CHECK( !check_response_code(pack, 302) );   // not a redirect!
     delete(pack);
 
+    // bad request (no dot in the hostname), fall back to the referer trick
+    // Take the host from the referer, since the bad host must have been
+    // part of the path. We have seen such strange links in CPXY-151
+    pack = make_package("http://localhost:9036/cproxydebug-15/"
+               "badhostname/page.html",1);
+    // Must have the content-bit (8) in debug, or we fake a regular OK response
+    add_request_header(pack, "Referer",
+         "http://localhost:9036/XXX/node102/100/"
+         "ref.host/ref.path/ref.to/ref.page.html");
+    cpx->process(*pack);
+
+    YAZ_CHECK( check_response_code(pack, 302) );
+    YAZ_CHECK( check_response_header(pack, "Location",
+          "http://localhost:9036/XXX/node102/100/"
+          "ref.host/badhostname/page.html") );
+    delete(pack);
+
     remove_dump_dir("./cf.100.dump");
 
     // More complex session. Now (as of 3.1.0) with new-form cookie lines.
@@ -748,7 +780,8 @@ void test1(void)
               "Set-Cookie two=TWO; Domain=.indexdata.com; \n"
               "Set-Cookie three=THREE; domain=other.indexdata.com\n"
               "Set-Cookie  mixedcase=true; domain=WwW.InDeXdAtA.cOm\n"
-              "SkipLink skippage2.com skiplink2.com\n" );
+              "SkipLink skippage2.com skiplink2.com\n"
+              "CproxyHeader request\n" );
     // The cookie called mixedcase is to check that the domain matching is
     // indeed case-insensitive (CP-3482)
     pack = process_url(cpx, "http://localhost:9036/XXX/cproxydebug-7/node102/"
@@ -766,6 +799,7 @@ void test1(void)
                                     "Basic c29tZXVzZXI6c2VjcmV0cGFzc3dvcmQ=") );
     YAZ_CHECK( check_any_request_header(pack,"X-Metaproxy-SkipLink",
                                         "skippage2.com skiplink2.com" ));
+    YAZ_CHECK( check_request_header(pack, "X-MK-Component", " "));
     
     delete(pack);
 
@@ -786,8 +820,8 @@ void test1(void)
     cpx->process(*pack);
     YAZ_CHECK( check_response_code(pack, 200) );
     YAZ_CHECK( check_no_request_header(pack, "Authorization") );
-    delete(pack);
-
+    delete(pack);    
+    
     // Session with new-form cookie lines, username without password
     // also a Basicauthwith, that forces auth to .dk and not to .com
     make_session_file( "." , "102",
@@ -795,6 +829,7 @@ void test1(void)
               "Set-Cookie newcookie2=v2;Domain=www.indexdata.com;Path=/;\n"
               "Set-Cookie newcookie3=v3;Domain=www.indexdata.com;Path=/staff;\n"
               "Set-Cookie newcookie4=v4;Domain=www.indexdata.com;Path=/about;\n"
+              "Set-Cookie infoweb=dot;Domain=.infoweb.newsbank.com;Path=/;\n"
               "Username someuser\n"
               "Baseurl http://www.indexdata.com/somepage?with=two&params=x\n"
               "Basicauthwith www.indexdata.dk\n"
@@ -820,6 +855,35 @@ void test1(void)
     YAZ_CHECK( check_request_header(pack, "Authorization", "Basic c29tZXVzZXI6") );
     delete(pack);
 
+    // CPXY-146: Request to foo.infoweb.newsbank.com. Should get the cookie. 
+    pack = make_package("http://localhost:9036/XXX/cproxydebug-verbose-nomove-dump/node102/"
+                        "102/foo.infoweb.newsbank.com/staff/",0);
+    cpx->process(*pack);
+    YAZ_CHECK( check_response_code(pack, 200) );
+    YAZ_CHECK( check_request_header(pack, "Cookie", "infoweb=dot;newcookie=newvalue") );
+    delete(pack);
+    // CPXY-146: Request to infoweb.newsbank.com (no leading dot). Should also get the cookie. 
+    pack = make_package("http://localhost:9036/XXX/cproxydebug-verbose-nomove-dump/node102/"
+                        "102/infoweb.newsbank.com/staff/",0);
+    cpx->process(*pack);
+    YAZ_CHECK( check_response_code(pack, 200) );
+    YAZ_CHECK( check_request_header(pack, "Cookie", "infoweb=dot;newcookie=newvalue") );
+    delete(pack);
+    
+    // Post request with a bad url, but with a referer header
+    // Should process it directly, and not redirect. Check session cookie path match CPXY-144
+    std::string postcontent = url_enc("foo=bar") + "&";
+    pack = make_package("http://localhost:9036/cproxydebug-nomove-verbose/"
+               "about/page.txt?param=value",1,"POST",postcontent);
+    add_request_header(pack, "Referer",
+         "http://localhost:9036/XXX/node102/102/"
+         "www.indexdata.com/sitemap/ref.page.html");
+    cpx->process(*pack);
+    YAZ_CHECK( !check_response_code(pack, 302) );   // not a redirect!
+    YAZ_CHECK( check_request_header(pack, "Cookie", // Check v4 is there, path matches
+            "newcookie4=v4;newcookie2=v2;newcookie=newvalue") );  // and v2, no path. but not v3
+    delete(pack);
+
     remove_dump_dir("./cf.102.dump");
 
 
@@ -999,6 +1063,16 @@ void test1(void)
     YAZ_CHECK( !check_request_header(pack, "Cookie",  "nocookie") );
     delete(pack);
 
+    pack = make_package("http://localhost:9036/XXX/cproxydebug-verbose-nomove-dump/node102/"
+                        "103/omit4.com/staff/",0);
+    add_request_header(pack, "X-Forwarded-Host","host.indexdata.com");
+    add_request_header(pack, "X-Forwarded-For","for.indexdata.com");
+    cpx->process(*pack);
+    YAZ_CHECK( check_no_request_header(pack, "X-Forwarded-Host") );
+    YAZ_CHECK( !check_no_request_header(pack, "X-Forwarded-For") );  
+    YAZ_CHECK( check_request_header(pack, "X-Request-Header","Added by cproxy" ) );
+    delete(pack);
+    
     remove_dump_dir("./cf.103.dump");
 
     delete(cpx);
@@ -1032,6 +1106,7 @@ void test2 (void)
         "<start route='start'/>\n"
         "<routes>\n"
             "<route id='start'>\n"
+
                 "<filter type='cproxy'>\n"
                     "<debug>0</debug>\n"    // will be enabled as needed
                     "<cfconfig>./cf-config.unittest2.cfg</cfconfig>\n"
@@ -1054,7 +1129,25 @@ void test2 (void)
                     "<skiplink>\n"
                         "<link>skiplink.com</link>\n"
                     "</skiplink>\n"
-                    
+                    "<skiprequestheader>\n"
+                    "  <page>sub.dummydomain.com</page>\n"
+                    "  <header>X-Funny-Request</header>\n"
+                    "</skiprequestheader>\n"
+                    "<addrequestheader>\n"
+                    "  <page>sub.dummydomain.com</page>\n"
+                    "  <header>X-Request-Header</header>\n"
+                    "  <value>Cproxy added this</value>\n"
+                    "</addrequestheader>\n"
+                    "<skipresponseheader>\n"
+                    "  <page>sub.dummydomain.com</page>\n"
+                    "  <header>X-Funny-Header</header>\n"
+                    "</skipresponseheader>\n"
+                    "<addresponseheader>\n"
+                    "  <page>sub.dummydomain.com</page>\n"
+                    "  <header>X-Response-Header</header>\n"
+                    "  <value>Cproxy added this</value>\n"
+                    "</addresponseheader>\n"
+                    "<cproxyheader>none</cproxyheader>\n"
                 "</filter>\n"
 
 #ifdef LOG_ALL
@@ -1231,7 +1324,12 @@ void test2 (void)
                 "CookieDomainExpansion .wc3.com foo bar\n"
                 "CookieDomainExpansion .wc4.com - foo bar\n"
                 "SkipLink file1.html skipped.html\n"
-                     );
+                "AddRequestHeader sub.dummydomain.com X-Request-Header-Ses Session Added This\n"
+                "SkipRequestHeader .*\\.dummydomain.com X-Strange-Request-Header\n"
+                "AddResponseHeader .* X-Response-Header-Ses Response via the session\n"
+                "SkipResponseHeader dummydomain.com X-Another-Funny-Header\n"
+                "CproxyHeader both\n"
+                );
     
     // Simple test with nomove, only invokes cproxy filter
     // Tests mostly that the other filters load all right
@@ -1242,6 +1340,7 @@ void test2 (void)
     pack->router(router).move();
     YAZ_CHECK( check_response_code(pack, 200) );
     YAZ_CHECK( check_response_content(pack, "got sessionfilename './cf.200'" ) );
+    YAZ_CHECK( check_request_header(pack,"X-MK-Component", " ") );
     delete(pack);
 
     // simple request going through the whole filter chain
@@ -1260,6 +1359,8 @@ void test2 (void)
     // the cproxysession cookie has the target host name, and the session number
     YAZ_CHECK( check_response_header(pack,"Set-Cookie",
                             "cproxysession=dummyhost.com/200; Path=/" ) );
+    YAZ_CHECK( check_request_header(pack,"X-MK-Component", " ") );
+    YAZ_CHECK( check_response_header(pack,"X-MK-Component", " ") );
     delete(pack);
 
     // Same as above, but without any debug info
@@ -1448,6 +1549,8 @@ void test2 (void)
       "Set-Cookie: w2=v; domain=.wc2.com\n" 
       "Set-Cookie: w3=v; domain=.wc3.com\n"
       "Set-Cookie: w4=v; domain=.wc4.com\n"
+      "X-Funny-Header: Ha ha\r\n"
+      "X-Another-Funny-Header: Hi hi\r\n"
         "\n"
       "<html><head>"
       "</head>\n"
@@ -1461,6 +1564,7 @@ void test2 (void)
             "cproxydebug-verbose-dump-keepcontent/"
             "YYY/ZZZ/200/"
             "sub.dummydomain.com/rawtestfile/rawfile.html",0);
+    add_request_header(pack, "X-Strange-Request-Header","Funny indeed");
     pack->router(router).move();
     YAZ_CHECK( check_response_code(pack, 200) );
     YAZ_CHECK( check_response_header(pack,"Content-Type", "text/html") );
@@ -1547,6 +1651,14 @@ void test2 (void)
                 "w4=v; domain=localhost; path=/YYY/ZZZ/200/www.wc4.com/") );
     YAZ_CHECK( check_any_response_header(pack,"Set-Cookie",
                 "w4=v; domain=localhost; path=/YYY/ZZZ/200/foo.wc4.com/") );
+    // Check the skip/add responseheader things from the config
+    YAZ_CHECK( check_request_header(pack, "X-Request-Header", "Cproxy added this" ) );
+    YAZ_CHECK( check_response_header(pack, "X-Response-Header", "Cproxy added this" ) );
+    YAZ_CHECK( check_no_response_header(pack, "X-Funny-Header" ) );
+    // These headers were modified because of the session
+    YAZ_CHECK( check_no_response_header(pack, "X-Another-Funny-Header" ) );
+    YAZ_CHECK( check_request_header(pack, "X-Request-Header-Ses", "Session Added This" ) );
+    YAZ_CHECK( check_response_header(pack, "X-Response-Header-Ses", "Response via the session" ) );
     
     delete(pack);
 
@@ -1587,6 +1699,9 @@ void test2 (void)
     // empty cookie
     YAZ_CHECK( check_response_content(pack,
                    "OK: name='e' value=''" ) );
+    // List of hostnames
+    YAZ_CHECK( check_response_content(pack,
+                   "sub.dummydomain.com 2 times" ) );
     // Post request not redirected (CPXY-81)
     YAZ_CHECK( !check_response_content(pack,
                "Directing to http://localhost:9036/YYY/ZZZ/200/"
@@ -1644,16 +1759,17 @@ void test2 (void)
         "Location: https://secure.com/login.html\r\n"
         "\r\n";
      
-     make_html_file( "./movedfile3", movedpage3 );
-     
-     pack = make_package("http://localhost:9036/"
-        "cproxydebug-verbose/"
-        "YYY/ZZZ/200/"
-        "dummyhost.com/rawtestfile/movedfile3",0);
-     pack->router(router).move();
-     YAZ_CHECK( check_response_code(pack, 400) );
-     delete(pack);
-     
+    make_html_file( "./movedfile3", movedpage3 );
+
+    pack = make_package("http://localhost:9036/"
+    "cproxydebug-verbose/"
+    "YYY/ZZZ/200/"
+    "dummyhost.com/rawtestfile/movedfile3",0);
+    pack->router(router).move();
+    YAZ_CHECK( check_response_code(pack, 400) );
+    delete(pack);
+
+
      
     // Test that the package can get to the bounce filter, and that the request
     // is properly deproxified at that point.
@@ -1694,8 +1810,8 @@ void test2 (void)
          "http://localhost:9036/XXX/node102/100/ref.host/ref.path");
     pack->router(router).move();
     YAZ_CHECK( check_response_code(pack, 400) );
-    delete(pack);
-
+    delete(pack);    
+    
     remove_dump_dir( "./cf.200.dump" );  
     
 } // test2
